"""Tools for interacting with databases in the context of alfred experiments. 

Authors: Christian Treffenstädt (treffenstaedt@psych.uni-goettingen.de) & Johannes Brachem (brachem@psych.uni-goettingen.de)

Alfred3 is a Python library for the generation of dynamic and interactive surveys and experiments. See `alfred3 on GitHub <https://github.com/ctreffe/alfred>`_ for more information on alfred3.
"""